/*
 * Software fr ATMega8 und kompatible
 * fr PWM-Board turi
 * Basissoftware ausgelegt fr:
 *    RC5-Fernbedienung (Port D2)
 *    4 PWM-Kanle mit Helligkeitsregelung fr 8-Bit PWM
 *    spter 4 ADC-Kanal fr Potis RGB/HSB optional ber Pin-Switch
 * 
 * Software-Komponenten verschiedener Foren verwendet und teilweise adaptiert (s. Doku
 * der jeweiligen Files)
 *
 * Fernbedienung vivanco UR12: TV1:102; SAT1:316
 *
 */


// Komponenten fr RC5 und ADC 
#include "RC5_v1.c"
//#include "adc.c"

#include <avr/pgmspace.h>
#include <avr/eeprom.h> 
#include <util/delay.h>

//EPROM-Speicher
uint16_t ee_pwm1 EEMEM = 255; //Red
uint16_t ee_pwm2 EEMEM = 127; //Green
uint16_t ee_pwm3 EEMEM = 0; //Blue
uint16_t ee_pwm4 EEMEM = 30; //brightness
uint16_t ee_pwm5 EEMEM = 127; //sat
uint16_t ee_pwm6 EEMEM = 10; //hue
uint8_t ee_pwm7 EEMEM = 6;  //Programm
uint16_t ee_pwm8 EEMEM = 10;  //speed
uint16_t ee_pwm9 EEMEM = 50; //White
 
uint16_t bri_table_8[64]  PROGMEM = {0,0,1,1,1,2,2,2,2,2,2,3,3,3,3,4,4,4,5,5,6,
									6,7,8,8,9,10,11,12,13,14,15,17,18,20,22,24,
									26,28,31,34,37,40,44,48,52,57,62,68,74,81,89,
									97,106,116,126,138,150,164,179,196,214,234,255};


void
initPWM (void)
{
  //DDRB |= (1 << PB3);
  //DDRD |= (1 << PD3) | (1 << PD5) | (1 << PD6);
  ////////////////////////////////////////////////////////////////////////////
#if defined (__AVR_ATmega8__) || defined (__AVR_AT90S2313__) \
		|| defined (__AVR_ATmega32__) || defined (__AVR_ATmega16__)
  ////////////////////////////////////////////////////////////////////////////

  //8-Bit-PWM initialisieren - Timer 1
  TCCR1A = 0xA1;		//  8 Bit Phase correct PWM 
  TCCR1B |= (0 << CS12) | (1 << CS11) | (1 << CS10);			// Prescale 64 

  // Timer 2
  TCCR2 =
    (1 << COM21) | (0 << COM20) | (0 << WGM21) | (1 << WGM20) | (1 << CS22) |
    (0 << CS21) | (0 << CS20); // Prescale 64

  ////////////////////////////////////////////////////////////////////////////
#elif defined (__AVR_ATmega48__) || defined (__AVR_ATmega88__) \
		|| defined (__AVR_ATmega168__)
  ////////////////////////////////////////////////////////////////////////////

  //8-Bit-PWM initialisieren - Timer 1
  TCCR1A = 0xA1;		//  8 Bit Phase correct PWM 
  TCCR1B |= (0 << CS12) | (1 << CS11) | (1 << CS10);			// Prescale 64 

  // Timer 2
  TCCR2A =
    (1 << COM2A1) | (0 << COM2A0) | (1 << COM2B1) | (0 << COM2B0) 
    | (1 <<  WGM20);
  TCCR2B =  (1 << CS22) | (0 << CS21) | (0 << CS20);			// Prescale 64

  ////////////////////////////////////////////////////////////////////////////
#else
#   error This AVR derivative is (not yet) supported.
#   error Feel free to add the required code.
#endif // AVR derivatives
  ////////////////////////////////////////////////////////////////////////////
}

#define LEDr OCR1A
#define LEDg OCR1B
#define LEDb OCR2B
#define LEDw OCR2A


int main (void) {            

// OC1A=PB1, OC1B=PB2, OC2A=PB3, OC2B=PD3
// IR-Sensor PD2
 
   	DDRB  = 0b00001110;             
   	PORTB = 0b00001110;            
   	DDRD  = 0b00001000;             
   	PORTD = 0b00001000;            

	//Konfiguration Nutzung Input an PB6 (vorauss. kein Quarz)
	//DDRB &= ~( 1 << PB6 );
	//enable internal pullup PB6
	//PORTB |= 0b01000000;

   	//Werte des ADC
	//uint16_t ADCval, ADCvalC1, ADCvalC2, ADCvalC3, ADCvalC0 ;		
	//int ADCdiff=0;

	//RC5-Code, Adresse
	uint8_t code, addr;		
	uint8_t DeviceType = TV1;	
	uint8_t LEDaus=1;

	//Variablen fr Farben etc.
	int16_t hue, sat, bri, red, blue, green, white, bri_val, bri_val_old, red_val, green_val, blue_val, white_val;

	uint8_t key=9;		//Taste fr Farbkanalwahl (1,2,3,5); Taste 0 zum Einleiten Speichern
	uint8_t iProg=0;	//Programmart
	uint16_t speed=10;	//Fading-Delay
	bri_val_old = 0;	//letzte Helligkeit vor Ausschalten

	//6 - RGB manuell
	//7 - wei mit Helligkeit
	//8 - HSB manuell
	//9 - HSB-Fading-Programm

	initPWM();

	//Fernbedinung aktivieren
	rc5_init (RC5_ALL);

	/* Interrupts zulassen */
	sei();

	//PB6 liegt auf Masse
	//if ( ! (PINB & (1 << PB6)) )
	//{
	// Kanal 0,1 ADC einmalig lesen, um Startwerte zu setzen
	//	ADCval = ReadChannel(3);ADCvalC3=ADCval; /* MUX-Bits auf 0b0000 -> Channel 3 */
	//	ADCval = ReadChannel(2);ADCvalC2=ADCval; /* MUX-Bits auf 0b0000 -> Channel 2 */
	//	ADCval = ReadChannel(1);ADCvalC1=ADCval; /* MUX-Bits auf 0b0000 -> Channel 1 */
	//	ADCval = ReadChannel(0);ADCvalC0=ADCval; /* MUX-Bits auf 0b0000 -> Channel 0 */
	//}

	//EEPROM lesen und Werte setzen
	red_val=eeprom_read_word(&ee_pwm1);
	green_val=eeprom_read_word(&ee_pwm2);
	blue_val=eeprom_read_word(&ee_pwm3);
	bri_val=eeprom_read_word(&ee_pwm4);
	sat=eeprom_read_word(&ee_pwm5);
	hue=eeprom_read_word(&ee_pwm6);
	iProg=eeprom_read_byte(&ee_pwm7);
	speed=eeprom_read_word(&ee_pwm8);
	white_val=eeprom_read_word(&ee_pwm9);

	bri=pgm_read_word(bri_table_8+bri_val);

	//Hauptschleife
   	while(1) 
	{
			                
			if (rc5.flip>=0) 
		   {	//FB verarbeiten
		      	/* Ja, dann rc5.code merken und evtl. rc5.addr */
		      	code = rc5.code;
		      	addr = rc5.addr;
		      	/* und auf naechstes Zeichen warten */
				if (addr==DeviceType)		//TV-Modus etc.
				{
					switch (code)
					{
					case 12: { 	
								code=0;					
								if (LEDaus==1) //ausschalten
								{
									bri_val_old = bri_val;
									bri_val=0; 
									LEDaus=0;

								}
								else				// wieder auf vorhergehenden Wert einschalten
								{
									bri_val=bri_val_old; 
									bri_val_old = 0;
									LEDaus=1;
								}
								_delay_ms(500);
								break;
							}    // Aus/wieder ein

		    		case 60: { 	if (iProg==7) { white_val=0; }
								if (iProg==8) { hue=0; } //rot
								if (iProg==6) { red_val=0; }				
								break;
							}    // Record

		    		case 46: { 	if (iProg==7) { bri_val=63;white_val=255; red_val=0;green_val=0;blue_val=0; }
								if (iProg==8) { hue=255; } //grn
								if (iProg==6) { red_val=255; }
								break;
							}    // back

		    		case 43: { 	if (iProg==7) { red_val=0;green_val=0;blue_val=0;white_val=0; }
								if (iProg==8) { red_val=0;green_val=0;blue_val=0;white_val=255;  } //nur ww
								if (iProg==6) { green_val=0; }
								if (iProg==9) { white_val=0; }					
								break;
							}    // Play

		    		case 44: { 	if (iProg==7) { bri_val=63;red_val=255;green_val=255;blue_val=245;white_val=255; }
								if (iProg==8) { hue=382;white_val=255; }  //gelb + ww
								if (iProg==6) { green_val=255; }					
								if (iProg==9) { white_val=80; }					
								break;
							}    // Stop

		    		case 56: { 	if (iProg==7) { red_val=0;green_val=0;blue_val=0; }
								if (iProg==8) { hue=382; white_val=0;  } //gelb
								if (iProg==6) { blue_val=0; }
								if (iProg==9) { speed=10; }					
								break;
							}    // exit

		    		case 42: { 	if (key==0)
								{	//Speichern der akt. Werte eingeleitet
									eeprom_write_word(&ee_pwm1,red_val ); //rot speichern
									eeprom_write_word(&ee_pwm2,green_val ); //grn speichern
									eeprom_write_word(&ee_pwm3,blue_val ); //blau speichern
									eeprom_write_word(&ee_pwm4,bri_val ); //akt. Helligkeit speichern
									eeprom_write_word(&ee_pwm5,sat ); //sat speichern
									eeprom_write_word(&ee_pwm6,hue ); //hue speichern
									eeprom_write_byte(&ee_pwm7,iProg ); //Programm speichern
									eeprom_write_word(&ee_pwm8,speed ); //speed speichern
									eeprom_write_word(&ee_pwm9,white_val ); //speed speichern
									//eeprom_busy_wait();
									//Besttigung kurz blinken
									LEDw = 20;_delay_ms(100);
									LEDw = 160;_delay_ms(100);
									LEDw = 20;_delay_ms(100);
									LEDw = 160;_delay_ms(100);
									_delay_ms(20);
									key=9;
								}
								else
								{	if (iProg==7) { bri_val=63;red_val=255;green_val=255;blue_val=245;white_val=0; }
									if (iProg==8) { hue=509;white_val=0;  } //blau
									if (iProg==6) { blue_val=255; }
								}
								break;
							}    // forward

		    		case 53: { 	if (iProg==7) { bri_val=53;red_val=255;green_val=255;blue_val=245;white_val=255; }
								if (iProg==8) { hue=127; white_val=0; }
								if (iProg==6) { white_val=0; }
								if (iProg==9) { white_val=160; }					
								break;
							}    // Pause

		    		case 13: { 	if (iProg==7) { bri_val=43;red_val=255;green_val=255;blue_val=250;white_val=255; }
								if (iProg==8) { hue=127; white_val=200; }
								if (iProg==6) { white_val=255; }
								if (iProg==9) { white_val=255; }					
								break;
							}    // Mute

		    		case 16:// Vol + in Abhngigkeit Zifferntaste
					 		{ 
								if (key==1)
								{
									if ((iProg==7) || (iProg==6))
									{
										red_val+=5;
										if (red_val > 255) red_val = 255;
									}
									else if ((iProg==8) || (iProg==9))
									{
										sat+=4;
										if (sat > 127) sat = 127;
									}
								}
								if (key==2)
								{
									if ((iProg==7) || (iProg==6)) 
									{
										green_val+=5;
										if (green_val > 255) green_val = 255;
									}
									else if (iProg==8)
									{
										hue+=5;
										if (hue > 764) hue = 764;
									}
								}
								if (key==3)
								{
									if ((iProg==7) || (iProg==6))
									{
										blue_val+=5;
										if (blue_val > 255) blue_val = 255;
									}
									else if (iProg==9)
									{
										speed-=5;
										if (speed < 10) speed = 10;
									}
								}
								if (key==5)
								{
										white_val+=5;
										if (white_val > 255) white_val = 255;
								}
								break;
							}    
		    		case 17:// Vol - in Abhngigkeit Zifferntaste
							 { 
								if (key==1)
								{
									if ((iProg==7) || (iProg==6)) 
									{
										red_val-=5;
										if (red_val < 0) red_val = 0;
									}
									else if ((iProg==8) || (iProg==9))
									{
										sat-=4;
										if (sat < 0) sat = 0;
									}
								}
								if (key==2)
								{
									if ((iProg==7) || (iProg==6)) 
									{
										green_val-=5;
										if (green_val < 0) green_val = 0;
									}
									else if (iProg==8)
									{
										hue-=5;
										if (hue < 0) hue = 0;
									}
								}
								if (key==3)
								{
									if ((iProg==7) || (iProg==6))
									{
										blue_val-=5;
										if (blue_val < 0) blue_val = 0;
									}
									else if (iProg==9)
									{
										speed+=5;
										if (speed > 200) speed = 200;
									}
								}
								if (key==5)
								{
										white_val-=5;
										if (white_val < 0) white_val = 0;
								}
								break;

							}    
		    		case 32: { // Ch + 
									bri_val+=2;
									if (bri_val > 63) bri_val = 63;
								break;
							}    
		    		case 33: { // Ch - 
									bri_val-=2;
									if (bri_val < 0) bri_val = 0;
								break;
							}    

					case 1:	{	key=1; break;}		//Farbe 1 rot
					case 2:	{	key=2; break;}		//Farbe 2 grn
					case 3:	{	key=3; break;}		//Farbe 3 blau
					case 5:	{	key=5; break;}		//Farbe 5 wei
					case 6:	{	iProg=6; break;}	//Prog wei	
					case 7:	{	iProg=7; break;}	//Prog wei	
					case 9: {	iProg=9; break;}	//Prog HSB-Fading
					case 8: {	iProg=8; break;}	//Prog HSB manuell
					case 0: {	key=0; break;}	//Speichern einleiten
		    		//default: ;    

					} //switch (code)


				} //addr==0

		      	rc5.flip = -1;

			} //FB-Verarbeitung
		

		//Helligkeit ber Lookup-Tabelle berechnen
		bri=pgm_read_word(bri_table_8+bri_val);  //0..255

		//HSB-Fading-Programm
		if (iProg==9)		
		{		   
			hue++;
			//Verzgerung
			_delay_ms(speed);
		}
		//HSB-Werte aus FB interpretieren 
		if ((iProg==8) || (iProg==9))
		{		   
			if (hue > 764) hue = 0;

			if (hue < 255) {
			   red_val = (10880 - sat * (hue - 170)) >> 7;
			   green_val = (10880 - sat * (85 - hue)) >> 7;
			   blue_val = (10880 - sat * 85) >> 7;
			}
			else if (hue < 510) {
			   red_val = (10880 - sat * 85) >> 7;
			   green_val = (10880 - sat * (hue - 425)) >> 7;
			   blue_val = (10880 - sat * (340 - hue)) >> 7;
			}
			else {
			   red_val = (10880 - sat * (595 - hue)) >> 7;
			   green_val = (10880 - sat * 85) >> 7;
			   blue_val = (10880 - sat * (hue - 680)) >> 7;
			}
		}
		// Ausgabe der Werte
		//im HSB-Programm geht Helligkeit bis 255 (*4)
		red = ((bri + 1) * red_val) >> 8;
		green = ((bri + 1) * green_val) >> 8;
		blue = ((bri + 1) * blue_val) >> 8;		   
		white = ((bri + 1) * white_val) >> 8;		   
		LEDr = red;
		LEDg = green;
		LEDb = blue;		
		LEDw = white;		


   }    //while                     
 
   return 0;                 
}
